// -------------------------------------------->>
// classicSnake
//
// Developed by: Alexandre Martins (@leX)
//  2003
// www.gamingbrasil.net
// =============================
// game.cpp
// Object "game"
// -------------------------------------------->>

#include "classicSnake.h"

_game game;


// ---------------------->>
// init()
// Initialize functions
// ---------------------->>
void _game :: init(){

    // playing
    playing=true;

    // Initializing Allegro
    allegro_init();
    
    // Installing keyboard, mouse and timer
    install_keyboard();
    install_mouse();
    install_timer();
    
    // Timer
    LOCK_VARIABLE(timer);
    LOCK_FUNCTION(addToTimer);
    install_int_ex(addToTimer,BPS_TO_TIMER(60));

    // Installing sound
    if(install_sound(DIGI_AUTODETECT,MIDI_AUTODETECT,NULL) != 0){
        allegro_message("install_sound() error: %s",allegro_error);
        allegro_message("No sound will be played");
    }
    
    // Changing graphic mode
    set_color_depth(16);
    if(set_gfx_mode(GFX_AUTODETECT_FULLSCREEN,640,480,0,0) != 0){
        allegro_message("set_gfx_mode(WINDOWED) error: %s",allegro_error);
        allegro_message("Trying other display mode...");
        if(set_gfx_mode(GFX_SAFE,640,480,0,0)!=0){
                allegro_message("set_gfx_mode(SAFE) error: %s",allegro_error);
                allegro_message("Couldn\'t change graphic mode.");
                allegro_exit();
                exit(1);
        }
    }
    
    // Window title
    char temp[30];
    sprintf(temp,"classicSnake v%s",VERSION);
    set_window_title(temp);
    
    // Creating a buffer
    buffer=create_bitmap(SCREEN_W,SCREEN_H);
    clear_bitmap(buffer);
    
    // Randomize
    randomize();
    
}



// ---------------------->>
// close()
// Exit functions
// ---------------------->>
void _game :: close(){

    // Deleting food
    food.destroy();
    
    // Deleting snake
    snake.destroy();
    
    // Deleting current stage
    stage.deleteStage();
    
    // Destroying the buffer
    destroy_bitmap(buffer);
    
    // Closing Allegro
    allegro_exit();
    
}



// ------------------------->>
// forceExit()
// Forces the game to exit
// ------------------------->>
void _game :: forceExit(int n){

    close();
    exit(n);
    
}




// ------------------------------>>
// askExit()
// Do you really want to exit?
// ------------------------------>>
void _game :: askExit(){

    BITMAP *temp, *exitScreen, *quit_box;
    char tempstr[30];
    
    // creating/loading bitmaps
    strcpy(tempstr,"");
    sprintf(tempstr,"%s\\quit_box.pcx",GFX_DIR);
    quit_box=load_bitmap(tempstr,default_palette);
    
    temp=create_bitmap(SCREEN_W,SCREEN_H);
    exitScreen=create_bitmap(SCREEN_W,SCREEN_H);
    clear_to_color(temp,makecol(0,0,0));
    clear_bitmap(exitScreen);
    
    // drawing a transparent background
    blit(buffer,exitScreen,0,0,0,0,SCREEN_W,SCREEN_H);
    set_trans_blender(0,0,0,128);
    draw_trans_sprite(exitScreen,temp,0,0);
    
    // drawing a box
    draw_sprite(exitScreen,quit_box,(SCREEN_W/2)-(quit_box->w/2),(SCREEN_H/2)-(quit_box->h/2));
    
    // updating screen
    blit(exitScreen,screen,0,0,0,0,SCREEN_W,SCREEN_H);
    
    // destroying bitmaps
    destroy_bitmap(quit_box);
    destroy_bitmap(temp);
    destroy_bitmap(exitScreen);
    
    // Waiting for user's input
    while(!key[KEY_Y] && !key[KEY_N]){
        if(timer>0) timer--;
    }
    if(key[KEY_Y]){
      // Deleting objects
      food.destroy();
      snake.destroy();
      stage.deleteStage();
      
      // Fade-out
      highcolor_fade_out(8);
      
      // Going to Introduction
      global[GV_FRAME]=FRAME_INTRO;
    }
}




// ---------------------->>
// drawScreen()
// Draws the screen
// ---------------------->>
void _game :: drawScreen(){

    blit(buffer,screen,0,0,0,0,SCREEN_W,SCREEN_H);
    
}




// -------------------------->>
// randomize()
// Randomize
// -------------------------->>
void _game :: randomize(){

    srand(time(NULL));
    
}



// ---------------------------->>
// random()
// Returns a random number
// ---------------------------->>
int _game :: random(int num){

    return rand()%(num);
    
}



// ------------------------------------------>>
// readStr()
// reads a string in fp between two '\"'
// ------------------------------------------>>
void _game :: readStr(FILE *fp, char str[]){

    bool isReading=false;
    char c='\0';
    int a=0;
    
    while(c!=EOF){
        if(c=='\"' && isReading) break;
        if(c=='\"' && !isReading) isReading=true;
        
        c=getc(fp);
        if(isReading){
                str[a]=c;
                a++;
        }
    }
    if(c!=EOF) str[a-1]='\0';
    
}





// ----------------------->>
// highcolor_fade_out
// fade out effect
// ----------------------->>
void _game :: highcolor_fade_out(int speed)
{
    BITMAP *bmp_orig, *bmp_buff;

    if ((bmp_orig = create_bitmap(SCREEN_W, SCREEN_H)))
    {
        if ((bmp_buff = create_bitmap(SCREEN_W, SCREEN_H)))
        {
            int a;
            blit(screen, bmp_orig, 0,0, 0,0, SCREEN_W, SCREEN_H);
            if (speed <= 0) speed = 16;
        
            for (a = 255-speed; a > 0; a-=speed)
            {
                while(timer>0) timer--;
                clear_bitmap(bmp_buff);
                set_trans_blender(0,0,0,a);
                draw_trans_sprite(bmp_buff, bmp_orig, 0, 0);
                blit(bmp_buff, screen, 0,0, 0,0, SCREEN_W, SCREEN_H);
            }
            destroy_bitmap(bmp_buff);
        }
        destroy_bitmap(bmp_orig);
    }

    rectfill(screen, 0,0, SCREEN_W,SCREEN_H, makecol(0,0,0));
}



// ------------------->>
// intro()
// Introduction
// ------------------->>
void _game :: intro(){

    bool runOnce=false, done=false, menuPressedKey=false;
    int titlebg2_scrollx=0, titlebg2_scrolly=0, menuop=0;
    char tempstr[30];
    MIDI *mus;
    BITMAP *title, *titlebg, *titlebg2, *menu, *mini_snake;
    SAMPLE *menusnd;
    
    while(!done){
       // ** Run once **
       if(!runOnce){
          // Loading and playing music
          sprintf(tempstr,"%s\\title.mid",MUSIC_DIR);
          if(!(mus=load_midi(tempstr))){
                    allegro_message("File \"%s\" not found",tempstr);
                    forceExit(1);
          }
          play_midi(mus,true);
          
          // Loading menusnd
          sprintf(tempstr,"%s\\food.wav",SFX_DIR);
          if(!(menusnd=load_sample(tempstr))){
                    allegro_message("File \"%s\" not found",tempstr);
                    forceExit(1);
          }
          
          // Loading title image
          sprintf(tempstr,"%s\\title.pcx",GFX_DIR);
          if(!(title=load_bitmap(tempstr,default_palette))){
                    allegro_message("File \"%s\" not found",tempstr);
                    forceExit(1);
          }
          
          // Loading titlebg
          sprintf(tempstr,"%s\\titlebg.pcx",GFX_DIR);
          if(!(titlebg=load_bitmap(tempstr,default_palette))){
                    allegro_message("File \"%s\" not found",tempstr);
                    forceExit(1);
          }
          
          // Loading menu
          sprintf(tempstr,"%s\\menu.pcx",GFX_DIR);
          if(!(menu=load_bitmap(tempstr,default_palette))){
                    allegro_message("File \"%s\" not found",tempstr);
                    forceExit(1);
          }
          
          // Loading mini_snake
          sprintf(tempstr,"%s\\mini_snake.pcx",GFX_DIR);
          if(!(mini_snake=load_bitmap(tempstr,default_palette))){
                    allegro_message("File \"%s\" not found",tempstr);
                    forceExit(1);
          }
          
          // Creating titlebg2
          titlebg2=create_bitmap(SCREEN_W,SCREEN_H);
          clear_bitmap(titlebg2);
          for(int y=0;y<(SCREEN_H/titlebg->h)+1;y++){
             for(int x=0;x<(SCREEN_W/titlebg->w)+1;x++){
                blit(titlebg,titlebg2,0,0,x*titlebg->w,y*titlebg->h,titlebg->w,titlebg->h);
             }
          }
          
          // runOnce
          runOnce=true;
       }
       
       // ** Menu **
       if(key[KEY_UP] && !menuPressedKey){ menuop--; menuPressedKey=true; play_sample(menusnd,255,128,1000,false); }
       if(key[KEY_DOWN] && !menuPressedKey){ menuop++; menuPressedKey=true; play_sample(menusnd,255,128,1000,false); }
       if(!key[KEY_UP] && !key[KEY_DOWN]){ menuPressedKey=false; }
       if(menuop<0) menuop=2;
       if(menuop>2) menuop=0;
       
       // ** Drawing **
       // titlebg2
       for(int y=0;y<2;y++){
          for(int x=0;x<2;x++){
            blit(titlebg2,buffer,0,0,(x*titlebg2->w)-titlebg2_scrollx,(y*titlebg2->h)-titlebg2_scrolly,titlebg2->w,titlebg2->h);
          }
       }
       if(titlebg2_scrollx<titlebg2->w) titlebg2_scrollx++; else titlebg2_scrollx=0;
       if(titlebg2_scrolly<titlebg2->h) titlebg2_scrolly++; else titlebg2_scrolly=0;
       
       // credits
       rectfill(buffer,0,SCREEN_H-22,SCREEN_W,SCREEN_H,makecol(0,0,0));
       textprintf_centre_ex(buffer,font,SCREEN_W/2,SCREEN_H-20,makecol(255,255,255),-1,"classicSnake v%s - made by Alexandre Martins (@leX) - 2003",VERSION);
       textprintf_centre_ex(buffer,font,SCREEN_W/2,SCREEN_H-10,makecol(255,225,0),-1,"http://www.gamingbrasil.net");
       
       // title
       draw_sprite(buffer,title,(SCREEN_W/2)-(title->w/2),10);
       
       // menu
       draw_sprite(buffer,menu,(SCREEN_W/2)-(menu->w/2),(SCREEN_H/2)-(menu->h/2));
       if(menuop==0) draw_sprite(buffer,mini_snake,((SCREEN_W/2)-(menu->w/2))+10,((SCREEN_H/2)-(menu->h/2))+40);
       if(menuop==1) draw_sprite(buffer,mini_snake,((SCREEN_W/2)-(menu->w/2))+10,((SCREEN_H/2)-(menu->h/2))+85);
       if(menuop==2) draw_sprite(buffer,mini_snake,((SCREEN_W/2)-(menu->w/2))+10,((SCREEN_H/2)-(menu->h/2))+130);
       
       // screen
       drawScreen();
       
       // Option
       if(key[KEY_ENTER]){
              if(menuop==0) done=true;
              if(menuop==1) system("notepad readme.txt");
              if(menuop==2){ done=true; playing=false; }
       }
    }
    
    stop_midi();
    destroy_midi(mus);
    destroy_sample(menusnd);
    destroy_bitmap(title);
    destroy_bitmap(titlebg);
    destroy_bitmap(titlebg2);
    destroy_bitmap(menu);
    destroy_bitmap(mini_snake);
    
    highcolor_fade_out(32);
    
    // If game is over
    if(!playing){
      destroy_bitmap(buffer);
      allegro_exit();
      exit(0);
    }

}




// ----------------->>
// gameClear()
// Game clear
// ----------------->>
void _game :: gameClear(){
    bool done, keepTrying_newline;
    int dMsg_timer=0, dMsg_pos=0, dMsg_px=0, dMsg_py=0, dMsg_x=0, dMsg_y=0;
    int boxWidth=300, boxHeight=100;
    string msg, dMsg;
    BITMAP *gc, *box;
    char tempstr[30];

    // -----
    
    // ** Starting... **
    sprintf(tempstr,"%s\\gc.pcx",GFX_DIR);
    gc=load_bitmap(tempstr,default_palette);
    
    clear_bitmap(buffer);
    
    box=create_bitmap(boxWidth,boxHeight);
    clear_to_color(box,makecol(0,0,255));
    rectfill(box,2,2,box->w-2,box->h-2,makecol(0,100,255));
    
    msg="Congratulations! You\'ve cleared classicSnake! Thanks for playing. It was made by me - Alexandre Martins (@leX). This is my first game written in C++";
    
    // -----

    // ** Destroying some objects... **
    snake.destroy();
    food.destroy();
    stage.deleteStage();
    
    // -----
    
    // ** Loop **
    while(!done){
        // Timer
        while(timer>0) timer--;
        
        // Clearing buffer
        clear_bitmap(buffer);
        
        // Text effect
        dMsg_timer++;
        if(dMsg_timer>=4){
                dMsg_timer=0;
                if(dMsg_pos<msg.length()) dMsg_pos++;
                dMsg_px++;
        }
        if(dMsg_pos>=msg.length()){
                if(key[KEY_ENTER]) done=true;
                textprintf_centre_ex(buffer,font,SCREEN_W/2,SCREEN_H-20,makecol(255,225,0),-1,"Press [ENTER] to continue...");
        }
        
        // Drawing
        draw_sprite(buffer,gc,(SCREEN_W/2)-(gc->w/2),10);
        
        // Text - printing a SINGLE character
        if(dMsg_pos<msg.length()){
                dMsg=msg.substr(dMsg_pos,1);
                dMsg_x=10+(dMsg_px*(text_length(font,dMsg.data())+2));
                dMsg_y=10+(dMsg_py*(text_height(font)+2));
        
                if(dMsg_px==22 && dMsg==" "){ dMsg_py++; dMsg_px=0; dMsg_pos++; }
                if(dMsg_px==22 && dMsg!=" "){ keepTrying_newline=true; }
                if(keepTrying_newline && dMsg==" "){ dMsg_py++; dMsg_px=0; dMsg_pos++; keepTrying_newline=false; }
        
                textprintf_ex(box,font,dMsg_x,dMsg_y,makecol(255,255,255),-1,"%s",dMsg.data());
        }    
        
        // Drawing screen
        blit(box,buffer,0,0,(SCREEN_W/2)-(box->w/2),200,box->w,box->h);
        blit(buffer,screen,0,0,0,0,SCREEN_W,SCREEN_H);
    }
    
    // -----
    
    destroy_bitmap(gc);
    destroy_bitmap(box);
    
    highcolor_fade_out(8);
}
        





// -------------------------------------------------------------->>
// runWhileFalse()
// This will run once each time global[GV_FRAME]=FRAME_PLAYING
// -------------------------------------------------------------->>
void _game :: runWhileFalse(){

    // Setting some variables...
    global[GV_SCORE]=0;
    global[GV_ACTUAL_STAGE]=1;
    
    // Initializing objects
    snake.init();
    food.init();
    stage.loadStage(global[GV_ACTUAL_STAGE]);
    stage.playMusic();
    
}



// =========================================================>
// -------------------------------------------------------->>
//                         logicloop()
//                         Logic loop
// -------------------------------------------------------->>
// =========================================================>
void _game :: logicloop(){
    
    
    
    // **************************
    // The following events will run ONCE (MUST be the first event in this loop)
    // **************************
    if(!global[GV_RUN_ONCE]){
        global[GV_RUN_ONCE]=true;
        
        // Start of game: actual stage = first
        global[GV_ACTUAL_STAGE]=1;
    }
    
    
    
    // <<<<<<<<<<<<<<
    // **************
    // Logic loop
    // **************
    // >>>>>>>>>>>>>>
    
    // == Introduction ==
    // ==================
    if(global[GV_FRAME]==FRAME_INTRO){
    
        // run while false
        global[GV_RUN_WHILE_FALSE]=false;
        
        // Introduction
        intro();
        
        // Next frame...
        global[GV_FRAME]=FRAME_PLAYING;
    }
    
    
    // == Playing ==
    // =============
    if(global[GV_FRAME]==FRAME_PLAYING){
        // buffer
        clear_bitmap(buffer);
    
        // gv run while false
        if(!global[GV_RUN_WHILE_FALSE]){
                global[GV_RUN_WHILE_FALSE]=true;
                runWhileFalse();
        }
        
        // stage
        stage.drawBackground();
        
        // snake
        snake.move();
        snake.draw();
        
        // food
        food.draw();
        
        // status
        stage.drawStatus();
        stage.advance();
    }
    
    
    // == GameOver ==
    // ==============
    if(global[GV_FRAME]==FRAME_GAMEOVER){
        // Deleting objects
        snake.destroy();
        food.destroy();
        stage.deleteStage();
        
        // Text output
        rectfill(buffer,(SCREEN_W/2)-100,(SCREEN_H/2)-10,(SCREEN_W/2)+100,(SCREEN_H/2)+10,makecol(0,0,0));
        rectfill(buffer,(SCREEN_W/2)-98,(SCREEN_H/2)-8,(SCREEN_W/2)+98,(SCREEN_H/2)+8,makecol(255,0,0));
        textprintf_centre_ex(buffer,font,SCREEN_W/2,(SCREEN_H/2)-4,makecol(255,255,255),-1,"You are dead.");
        for(int a=0;a<450;a++){
                while(timer>0) timer--;
                blit(buffer,screen,0,0,0,0,SCREEN_W,SCREEN_H);
        }
        
        highcolor_fade_out(8);
        
        // goto introduction
        global[GV_FRAME]=FRAME_INTRO;
    }
    
    
    
    // == Game clear ==
    // ================
    if(global[GV_FRAME]==FRAME_CLEAR){
        highcolor_fade_out(8);
        gameClear();
        global[GV_FRAME]=FRAME_INTRO;
    }
    
    
    
    // **************************
    // Closing the game... (MUST be the last event in this loop)
    // **************************
    if(key[KEY_ESC]){
        askExit();
    }
}
