// -------------------------------------------->>
// classicSnake
//
// Developed by: Alexandre Martins (@leX)
//  2003
// www.gamingbrasil.net
// =============================
// snake.cpp
// Object "snake"
// -------------------------------------------->>

#include "classicSnake.h"

_snake snake;



// ------------------------>>
// init()
// Initializes the snake
// ------------------------>>
void _snake :: init(){

    // Position and direction
    x=(SCREEN_W/2)/SNAKE_WIDTH;
    y=(SCREEN_H/2)/SNAKE_HEIGHT;
    direction=game.random(4);
    
    // Bitmaps
    headBMP=create_bitmap(SNAKE_WIDTH,SNAKE_HEIGHT);
    bodyBMP=create_bitmap(SNAKE_WIDTH,SNAKE_HEIGHT);
    tailBMP=create_bitmap(SNAKE_WIDTH,SNAKE_HEIGHT);
    clear_to_color(headBMP,makecol(255,225,0));
    clear_to_color(bodyBMP,makecol(255,200,0));
    clear_to_color(tailBMP,makecol(255,175,0));
    
    // Speed
    if(speed<=0) speed=5;
    
    // Creating body
    firstBody=new _snakeBody;
    firstBody->prev=NULL;
    firstBody->next=NULL;
    for(int a=0;a<2;a++) createBody();
    
}






// --------------------------->>
// destroy()
// Destroys the snake
// --------------------------->>
void _snake :: destroy(){

    // Deleting bodys
    deleteBodys();

    // Destroying bitmaps
    destroy_bitmap(headBMP);
    destroy_bitmap(bodyBMP);
    destroy_bitmap(tailBMP);
    
}






// --------------------------->>
// createBody()
// Creates a new single body
// --------------------------->>
void _snake :: createBody(){

    _snakeBody *temp_actual, *temp_prev;
    
    temp_prev=NULL;
    temp_actual=firstBody;
    while(temp_actual!=NULL){
        temp_prev=temp_actual;
        temp_actual=temp_actual->next;
    }
    temp_actual=new _snakeBody;
    temp_actual->prev=temp_prev;
    temp_actual->next=NULL;
    temp_prev->next=temp_actual;
    
    temp_actual->x=temp_prev->oldBX;
    temp_actual->y=temp_prev->oldBY;

}






// -------------------------->>
// deleteBodys()
// Deletes all bodys
// -------------------------->>
void _snake :: deleteBodys(){

    _snakeBody *temp_actual, *temp_prev;
    
    temp_prev=NULL;
    temp_actual=firstBody;
    while(temp_actual!=NULL){
        temp_prev=temp_actual;
        temp_actual=temp_actual->next;
        delete temp_prev;
    }
    
}






// -------------------->>
// move()
// Moves the snake
// -------------------->>
void _snake :: move(){

    // Getting the direction (user input)
    if(key[KEY_UP]) direction=SNAKE_DIR_UP;
    if(key[KEY_RIGHT]) direction=SNAKE_DIR_RIGHT;
    if(key[KEY_DOWN]) direction=SNAKE_DIR_DOWN;
    if(key[KEY_LEFT]) direction=SNAKE_DIR_LEFT;
    
    // "snake" collides to "food"?
    if(food.hitTest(x,y)){
        createBody();
        food.changePosition();
        play_sample(food.eat,255,128,1000,false);
        game.global[GV_SCORE]+=10;
    }
        
        
    // "head" collides to "body"?
    if(bodyHitTest()){
        game.global[GV_FRAME]=FRAME_GAMEOVER;
    }   
    
     
       
    // ** snakeTimer (REAL movement) related **    
    snakeTimer++;
    if(snakeTimer>=speed){
    
        // Old head position
        oldHX=x;
        oldHY=y;
        
        // Moving body
        moveBody();
        
        // Moving tail
        snakeTail.x=gotoX;
        snakeTail.y=gotoY;
        
        
        // Moving
        if(direction==SNAKE_DIR_UP) y--;
        if(direction==SNAKE_DIR_RIGHT) x++;
        if(direction==SNAKE_DIR_DOWN) y++;
        if(direction==SNAKE_DIR_LEFT) x--;
        
        
        // If head touches some of the four borders of the screen
        if(x<0) x=(SCREEN_W/SNAKE_WIDTH)-1;
        if(x>(SCREEN_W/SNAKE_WIDTH)-1) x=0;
        if(y<0) y=((SCREEN_H/SNAKE_HEIGHT)-1)-2;
        if(y>((SCREEN_H/SNAKE_HEIGHT)-1)-2) y=0;

        
        // Reseting snakeTimer
        snakeTimer=0;
    }
    
}



// ---------------------------->>
// moveBody()
// Moves the body of the snake
// ---------------------------->>
void _snake :: moveBody(){

    _snakeBody *temp_actual, *temp_prev;

    // ** Moving **
    temp_prev=NULL;
    temp_actual=firstBody;
    while(temp_actual!=NULL){
    
        // Getting future position of the current body
        if(temp_prev==NULL){
            gotoX=oldHX;
            gotoY=oldHY;
        }else{
            gotoX=temp_prev->oldBX;
            gotoY=temp_prev->oldBY;
        }
        
        // Storing old position
        temp_actual->oldBX=temp_actual->x;
        temp_actual->oldBY=temp_actual->y;
        
        // Moving
        temp_actual->x=gotoX;
        temp_actual->y=gotoY;
        
        // Next...
        temp_prev=temp_actual;
        temp_actual=temp_actual->next;
        
    }
    
    // Storing old position (for tail movement)
    gotoX=temp_prev->oldBX;
    gotoY=temp_prev->oldBY;
}


// ------------------------>>
// draw()
// Draws the snake
// ------------------------>>
void _snake :: draw(){

    // Drawing body
    _snakeBody *temp_actual, *temp_prev;
    temp_prev=NULL;
    temp_actual=firstBody;
    while(temp_actual!=NULL){
        blit(bodyBMP,game.buffer,0,0,temp_actual->x*SNAKE_WIDTH,temp_actual->y*SNAKE_HEIGHT,SNAKE_WIDTH,SNAKE_HEIGHT);
        temp_prev=temp_actual;
        temp_actual=temp_actual->next;
    }
    
    // Drawing tail
    blit(tailBMP,game.buffer,0,0,snakeTail.x*SNAKE_WIDTH,snakeTail.y*SNAKE_HEIGHT,SNAKE_WIDTH,SNAKE_HEIGHT);    
    
    
    // Drawing head
    blit(headBMP,game.buffer,0,0,x*SNAKE_WIDTH,y*SNAKE_HEIGHT,SNAKE_WIDTH,SNAKE_HEIGHT);
    
}




// ----------------------------->>
// bodyHitTest()
// Checks for collision (body)
// ----------------------------->>
bool _snake :: bodyHitTest(){

    _snakeBody *temp_actual, *temp_prev;
    
    temp_prev=NULL;
    temp_actual=firstBody;
    while(temp_actual!=NULL){
        if(x==temp_actual->x && y==temp_actual->y) return true;
        temp_prev=temp_actual;
        temp_actual=temp_actual->next;
    }
    return false;
    
}
