// -------------------------------------------->>
// classicSnake
//
// Developed by: Alexandre Martins (@leX)
//  2003
// www.gamingbrasil.net
// =============================
// stage.cpp
// Object "stage"
// -------------------------------------------->>

#include "classicSnake.h"

_stage stage;

// --------------------------->>
// loadStage()
// Loads an specific stage
// --------------------------->>
void _stage :: loadStage(int s){

    FILE *fp;
    char tempstr[30],bgfile[30],musfile[30],stage_clearfile[30];
    int actual_stage=1;
    
    strcpy(tempstr,"");
    sprintf(tempstr,"%s\\%s",STAGE_DIR,STAGE_CFGFILE);
    
    if(!(fp=fopen(tempstr,"r"))){
        allegro_message("File \"%s\" not found");
        game.forceExit(1);
    }
    
    fscanf(fp,"number of stages: %d",&number_of_stages);
    if(s>number_of_stages) s=number_of_stages;
    if(s<1) s=1;
    
    while(actual_stage<=s){
        game.readStr(fp,bgfile);
        game.readStr(fp,musfile);
        fscanf(fp," (%d,%d)",&snake.speed,&neededScore);
        actual_stage++;
    }
    
    fclose(fp);
    
    // -----
    
    strcpy(tempstr,bgfile);
    sprintf(tempstr,"%s\\%s",GFX_DIR,bgfile);
    strcpy(bgfile,tempstr);
    
    strcpy(tempstr,musfile);
    sprintf(tempstr,"%s\\%s",MUSIC_DIR,musfile);
    strcpy(musfile,tempstr);
    
    if(!(background=load_bitmap(bgfile,default_palette))){
        allegro_message("File \"%s\" not found",bgfile);
        game.forceExit(1);
    }
    
    if(!(music=load_midi(musfile))){
        allegro_message("File \"%s\" not found",musfile);
        game.forceExit(1);
    }
    
    // -----
    
    bgRepeat_x=(SCREEN_W/background->w)+1;
    bgRepeat_y=(SCREEN_H/background->h)+1;
    
}



// ----------------------------->>
// deleteStage()
// deletes the current stage
// ----------------------------->>
void _stage :: deleteStage(){

    destroy_bitmap(background);
    
    stop_midi();
    destroy_midi(music);
    
}





// --------------------------------------->>
// drawBackground()
// draws the background into the screen
// --------------------------------------->>
void _stage :: drawBackground(){

    for(int y=0;y<bgRepeat_y;y++){
        for(int x=0;x<bgRepeat_x;x++){
                blit(background,game.buffer,0,0,x*background->w,y*background->h,background->w,background->h);
        }
    }
    
}



// -------------------------------------->>
// playMusic()
// plays the music of the current stage
// -------------------------------------->>
void _stage :: playMusic(){

    midi_pos=0;
    play_midi(music,true);
    
}




// ---------------------->>
// drawStatus()
// draws status
// ---------------------->>
void _stage :: drawStatus(){

    // Rect
    rectfill(game.buffer,0,SCREEN_H-20,SCREEN_W,SCREEN_H,makecol(0,0,0));
    
    // Score
    textprintf_ex(game.buffer,font,10,(SCREEN_H-10)-4,makecol(255,255,255),-1,"Score: %d / %d",game.global[GV_SCORE],neededScore);
    
    // Stage
    textprintf_centre_ex(game.buffer,font,SCREEN_W/2,(SCREEN_H-10)-4,makecol(0,200,255),-1,"Stage %d",game.global[GV_ACTUAL_STAGE]);
    
    // Press "ESC" to quit
    textprintf_right_ex(game.buffer,font,SCREEN_W-10,(SCREEN_H-10)-4,makecol(255,255,255),-1," to quit");
    textprintf_right_ex(game.buffer,font,(SCREEN_W-10)-text_length(font," to quit"),(SCREEN_H-10)-4,makecol(0,225,0),-1,"\"ESC\"");
    textprintf_right_ex(game.buffer,font,(SCREEN_W-10)-text_length(font," \"ESC\" to quit"),(SCREEN_H-10)-4,makecol(255,255,255),-1,"Press");
}




// --------------------->>
// advance()
// goto next stage
// --------------------->>
void _stage :: advance(){

    if(game.global[GV_SCORE]>=neededScore){
        if(game.global[GV_ACTUAL_STAGE]+1<=number_of_stages){
                // next stage...
                
                // deleting stage
                deleteStage();
                
                // freezing the screen
                rectfill(game.buffer,(SCREEN_W/2)-100,(SCREEN_H/2)-10,(SCREEN_W/2)+100,(SCREEN_H/2)+10,makecol(0,200,255));
                rectfill(game.buffer,(SCREEN_W/2)-98,(SCREEN_H/2)-8,(SCREEN_W/2)+98,(SCREEN_H/2)+8,makecol(0,128,255));
                textprintf_centre_ex(game.buffer,font,SCREEN_W/2,(SCREEN_H/2)-4,makecol(255,255,255),-1,"Stage clear!");
                for(int a=0;a<300;a++){
                   while(timer>0) timer--;
                   blit(game.buffer,screen,0,0,0,0,SCREEN_W,SCREEN_H);
                }
                
                // creating/modifying some objects
                food.changePosition();
                
                // setting some variables
                game.global[GV_ACTUAL_STAGE]++;
                loadStage(game.global[GV_ACTUAL_STAGE]);
                playMusic();
                
        }else{
        
                // game clear
                game.global[GV_FRAME]=FRAME_CLEAR;
        }
    }
    
}
